﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WPFPrimesGenerator
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private BackgroundWorker worker;
        private List<int> result = new List<int>();
        private int primesCount = 0;
        private ObservableCollection<int> minValues = new ObservableCollection<int>(),
                                  maxValues = new ObservableCollection<int>();
        delegate void DisplayPrime(int prime,int percents);
        DisplayPrime myDelegate;
        public MainWindow()
        {
            InitializeComponent();
            minValues.Add(2);
            for(int i = 10; i <= 200; i += 10)
            {
                minValues.Add(i);
            }
            for(int i=1000;i<=1000000; i += 1000)
            {
                maxValues.Add(i);
            }

            Min.ItemsSource = minValues;
            Max.ItemsSource = maxValues;
            Max.SelectedItem = 1000;
            Min.SelectedItem = 2;

        }

        private void New_Click(object sender, RoutedEventArgs e)
        {
            Data.Text = "";
        }

        private void Save_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            if(dialog.ShowDialog()==true)
            {
                using (StreamWriter writer = new StreamWriter(dialog.FileName))
                {
                    writer.Write(Data.Text);
                }
            }

        }

        private void Exit_Click(object sender, RoutedEventArgs e)
        {
            Application.Current.Shutdown();
        }

        private void Start_Click(object sender, RoutedEventArgs e)
        {
            worker = new BackgroundWorker();
            worker.WorkerReportsProgress = true;
            worker.DoWork += CalculatePrimes;
            worker.ProgressChanged += ShowProgress;
            worker.RunWorkerCompleted += Completed;
            Start.IsEnabled = false;
            Data.Text = "";

            int[] data = { (int)Min.SelectedItem, (int)Max.SelectedItem };
            worker.RunWorkerAsync(data);
        }

        private void Completed(object sender, RunWorkerCompletedEventArgs e)
        {
            Data.Text += "\nCalculation Done\nPrimes count: "+primesCount;
            Start.IsEnabled = true;
        }

        private void ShowProgress(object sender, ProgressChangedEventArgs e)
        {
            progress.Value = e.ProgressPercentage;
            Data.Text += (int)e.UserState + " ";
            
            if(primesCount%20 == 0)
            {
                Data.Text += "\n";
            }
            
        
        }

        private void CalculatePrimes(object sender, DoWorkEventArgs e)
        {
            List<int> values = new List<int>();
            int[] limits = (int[])e.Argument;
            primesCount = 0;
            int min = limits[0];
            int max = limits[1];
            for(int i=min;i<max;i++)
            {
                if (isPrime(i))
                {
                    result.Add(i);
                    (sender as BackgroundWorker).ReportProgress((i - min) * 100 / (max - min),i);
                    primesCount++;
                    Thread.Sleep(0);
                }
            }
        }

        private void StartClasic_Click(object sender, RoutedEventArgs e)
        {
            Thread thread = new Thread(ThreadStartMethod);
            int low = (int)Min.SelectedValue;
            int high = (int)Max.SelectedValue;
            int[] limits = { low, high };
            Data.Clear();
            progress.Value = 0;

            thread.Start(limits);
        }

        private void ThreadStartMethod(object param)
        {
            int[] limits = (int[])param;
            myDelegate = new DisplayPrime(displayMethod);

            for(int i=limits[0];i<=limits[1]; i++)
            {
                if (isPrime(i))
                {
                    Dispatcher.Invoke(myDelegate, new object[] { i, (i-limits[0])*100/(limits[1]-limits[0])});
                }
            }
        }

        private void displayMethod(int prime,int percents)
        {
            Data.Text += prime + "\n";
            progress.Value = percents;
        }

        private bool isPrime(int value)
        {
            if (value == 2) return true;
            if (value % 2 == 0) return false;
            double max = Math.Sqrt(value);
            for(int i = 3; i <= max; i+=2)
            {
                if(value % i == 0)
                {
                    return false;
                }
            }
            return true;
        }
    }
}
